class GoogleTTS:

    def __init__(self):
        from gtts import gTTS
        self.gTTS = gTTS

    def __call__(self, text, outfile, lang='en'):
        tts = self.gTTS(text, lang=lang)
        tts.save(f'{outfile}.mp3')



import random
import re
import subprocess
import sys
from collections import defaultdict


class MAC:

    def __init__(self):
        import pandas as pd
        self.pd = pd
        self.speaker_profile = self.build_voices_profile()

    def build_voices_profile(self):
        profile_dict = defaultdict(list)
        output_bytes = subprocess.check_output(["say", "-v", "?"]).decode(sys.stdout.encoding)
        lang_list = [[p.strip() for p in re.split('  +', s)] for s in str(output_bytes).split("\n") if
                     len(s.strip()) > 0]
        for lang in lang_list:
            profile_dict[lang[1]].append(lang[0])
        return profile_dict

    def transform_into_mp3(self, fname):
        subprocess.check_output(
            ["ffmpeg", "-hide_banner", "-loglevel", "error", "-y", "-i", f"{fname}.aiff", "-f", "mp3", "-acodec",
             "libmp3lame", "-ab", "192000", "-ar",
             "44100", f"{fname}.mp3"])
        subprocess.call(['rm', f"{fname}.aiff"])

    # ref: https://gist.github.com/jeffeuxMartin/b96382cbee4a0e1eaa38b28925dc0b16
    def __call__(self, text, outfile, lang='en_US'):
        lang = lang.replace("-", "_")
        subprocess.check_output(
            ["say", '-v', random.choice(self.speaker_profile[lang]), f'{text}', '-o', f'{outfile}'])
        self.transform_into_mp3(outfile)



import requests
from requests.compat import urljoin


class AndroidTTS:

    def __init__(self, server_address):
        self.server_address = server_address

    def __call__(self, text, outfile, lang='en_US'):
        data = {'phrase': text, 'language': lang}
        response = requests.get(urljoin(self.server_address, 'tts.wav'), params=data)
        with open(f'{outfile}.wav', "wb") as f_out:
            f_out.write(response.content)



import random
from collections import defaultdict


class GoogleCloud:

    def __init__(self):
        from google.cloud import texttospeech
        self.texttospeech = texttospeech
        self.client = texttospeech.TextToSpeechClient()
        self.speaker_profile = self.build_voices_profile()

    def build_voices_profile(self):
        """Lists the available voices."""
        voices = self.client.list_voices()
        profile_dict = defaultdict(list)
        for voice in voices.voices:
            ssml_gender = self.texttospeech.SsmlVoiceGender(voice.ssml_gender)
            for language_code in voice.language_codes:
                profile_dict[language_code].append({
                    'speaker': voice.name,
                    'gender': ssml_gender.name,
                    'sr': voice.natural_sample_rate_hertz
                })
        return profile_dict

    def __call__(self, text, outfile, lang='en-US'):
        input_text = self.texttospeech.SynthesisInput(text=text)
        voice = self.texttospeech.VoiceSelectionParams(
            language_code=lang,
            name=random.choice(self.speaker_profile[lang])['speaker']
        )

        audio_config = self.texttospeech.AudioConfig(
            speaking_rate=random.uniform(0.8, 1.2),
            pitch=random.uniform(-1.7, 1.7),
            audio_encoding=self.texttospeech.AudioEncoding.MP3
        )

        response = self.client.synthesize_speech(
            request={"input": input_text, "voice": voice, "audio_config": audio_config}
        )

        with open(f'{outfile}.mp3', "wb") as out:
            out.write(response.audio_content)
